 /*Software Driver

SST25PF040C 4Mbit (512KB x 8) SPI

April 10, 2019 Rev. 1.1

ABOUT THE SOFTWARE
This application note provides software driver examples for  SST25PF040C
Serial Flash. Extensive comments are included in each routine to describe
the function of each routine.  The interface coding uses polling method
rather than the SPI protocol to interface with these serial devices.  The
functions are differentiated below in terms of the communication protocols
(uses Mode 0) and specific device operation instructions. 


ABOUT SST25PF040C

Companion product datasheets for the  SST25PF040C should be reviewed in
conjunction with this application note for a complete understanding
of the device.


Device Communication Protocol(pinout related) functions:

Functions                    		Function
------------------------------------------------------------------
init					Initializes clock to set up mode 0.
Send_Byte				Sends one byte using SI pin to send and
					shift out 1-bit per clock rising edge
Get_Byte				Receives one byte using SO pin to receive and shift
					in 1-bit per clock falling edge

Note:  The pin names of the    are used in this application note. The associated test code
will not compile unless these pinouts (SCK, SI, SO, WP,HOld, CE) are pre-defined on your
software which should reflect your hardware interfaced.

*/


//"C" LANGUAGE DRIVERS

/********************************************************************/
/* Copyright Microchip(SST), 1994-2015	    */
/* Example "C" language Driver of   SST25PF040C Serial Flash	    */
/* Hardik Patel, Microchip                  */
/*                                                                  */
/* Revision 1.0, Nov 23, 2015			  	    */
/*                                                                  */
/*								    */
/********************************************************************/

// include files
#include <p24fxxxx.h>

// defines

// configuration bits
        _CONFIG1(JTAGEN_OFF & ICS_PGx2 & FWDTEN_OFF)
		_CONFIG2(OSCIOFNC_ON &  FNOSC_FRCDIV & POSCMOD_NONE)

/* Pin Function Prototypes */
void Portassign();
void DisplayLED(char Led_Stat);
void CE_High(); 
void CE_Low();
void Hold_Low();
void Hold_High();
void WP_Low();
void WP_High();
void SCK_High(); 
void SCK_Low(); 
void SI_High(); 
void SI_Low(); 
void SO_High();
void SO_Low();
void SetSI_Output();
void SetSI_Input();
void SetSO_Output();
void SetSO_Input();
void init();
void Send_Byte(unsigned char out);
 void Send_Double_Byte(unsigned char out);
unsigned char Get_Byte();
unsigned char Get_Double_Byte();
unsigned char SPI_Read_Status_Register();
void SPI_WREN();
void SPI_WRDI();
void SPI_DPD();
void SPI_RDPD();
void NoOp();
unsigned char Read_ID();
void Jedec_ID_Read();
unsigned char SPI_Read(unsigned long Dst);
void SPI_Read_Cont(unsigned long Dst, unsigned long no_bytes);
unsigned char SPI_HighSpeed_Read(unsigned long Dst);
void SPI_HighSpeed_Read_Cont(unsigned long Dst, unsigned long no_bytes);
unsigned char SPI_Double_Output_Read(unsigned long Dst);
void SPI_Double_Output_Read_Cont(unsigned long Dst, unsigned long no_bytes);
unsigned char SPI_Double_IO_Read(unsigned long Dst);
void SPI_Double_IO_Read_Cont(unsigned long Dst, unsigned long no_bytes);
void SPI_Page_Program(unsigned long Dst);
void SPI_Chip_Erase();
void SPI_Sector_Erase(unsigned long Dst);
void SPI_Block_Erase(unsigned long Dst);
void SPI_NoOp();
void SPI_Write_Status_Register(unsigned int data1);
void SPI_Wait_Busy();


unsigned int data_256[256];	/* global array to store read data */
unsigned char   Manufacturer_Id, Device_Type, Device_Id; /* global array to store Manufacturer and Device id information */


/************************************************************************/
/* PROCEDURE:Port Assign												*/
/*																		*/
/* This procedure sets the LATs for SPI communication and LED Display.	*/
/* LATA is for LED display.  LATF is for SPI communication.			*/
/* Input:																*/
/*		None															*/
/*																		*/
/* Output:																*/
/*		None															*/
/*																		*/
/************************************************************************/					

void Portassign()
{
	LATA = 0x0000;		// set LAT A output latches low
	TRISA = 0x0000;		// set LAT A pins to be outputs

    LATG = 0xBC3F;
	TRISG =0xBC3F;  	//Set everything to be output.
	
    LATD = 0xFFFE;
	TRISD =0xFFFE;  	//Set everything to be output.

}

	

/************************************************************************/
/* PROCEDURE: Display LED												*/
/*																		*/
/* This procedure is used to diplay a byte on 8 LEDs which are 			*/
/* connected to LATA													*/
/* Input:																*/
/*		Led Display														*/
/*																		*/
/* Output:																*/
/*		Led Display														*/
/*																		*/
/************************************************************************/
void DisplayLED(char Led_Stat)
{
	LATA=Led_Stat;
}
	
	

/************************************************************************/
/* PROCEDURE: CE_High													*/
/*																		*/
/* This procedure set CE = High.										*/
/*																		*/
/* Input:																*/
/*		None															*/
/*																		*/
/* Output:																*/
/*		CE																*/
/*																		*/
/************************************************************************/
void CE_High()
{
	LATG = LATG | (0x0200);			/* set CE high */
}


/************************************************************************/
/* PROCEDURE: CE_Low													*/
/*																		*/
/* This procedure drives the CE of the device to low.  					*/
/*																		*/
/* Input:																*/
/*		None															*/
/*																		*/
/* Output:																*/
/*		CE																*/
/*																		*/
/************************************************************************/
void CE_Low() 
{	
	LATG = LATG & ~(0x0200);		/* clear CE low */

}

/************************************************************************/
/* PROCEDURE: Hold_Low()												*/
/*																		*/
/* This procedure clears the Hold pin to low.							*/
/*																		*/
/* Input:																*/
/*		None															*/
/*																		*/
/* Output:																*/
/*		Hold															*/
/************************************************************************/
void Hold_Low()
{
LATG = LATG & ~(0x4000);			/* clear Hold pin */
}

/************************************************************************/
/* PROCEDURE: Hold_High()												*/
/*																		*/
/* This procedure sets the Hold pin to high.							*/
/*																		*/
/* Input:																*/
/*		None															*/
/*																		*/
/* Output:																*/
/*		Hold															*/
/************************************************************************/
void Hold_High()
{
LATG = LATG | (0x4000);				/* set Hold pin */
}

/************************************************************************/
/* PROCEDURE: WP_Low()													*/
/*																		*/
/* This procedure clears the WP pin to low.								*/
/*																		*/
/* Input:																*/
/*		None															*/
/*																		*/
/* Output:																*/
/*		WP																*/
/************************************************************************/
void WP_Low()
{
LATD = LATD & ~(0x0001);			/* clear WP pin */
}

/************************************************************************/
/* PROCEDURE: WP_High()													*/
/*																		*/
/* This procedure sets the WP pin to high.								*/
/*																		*/
/* Input:																*/
/*		None															*/
/*																		*/
/* Output:																*/
/*		WP																*/
/************************************************************************/
void WP_High()
{
LATD = LATD | (0x01);				/* set WP pin */
}



/************************************************************************/
/* PROCEDURE: SCK_High													*/
/*																		*/
/* This procedure set CE = High.										*/
/*																		*/
/* Input:																*/
/*		None															*/
/*																		*/
/* Output:																*/
/*		SCK																*/
/*																		*/
/************************************************************************/
void SCK_High() 
{
	LATG = LATG | (0x40);			/* set SCK high */
}

/************************************************************************/
/* PROCEDURE: SCK_Low													*/
/*																		*/
/* This procedure drives the CLK of the device to low.  					*/
/*																		*/
/* Input:																*/
/*		None															*/
/*																		*/
/* Output:																*/
/*		SCK																*/
/*																		*/
/************************************************************************/
void SCK_Low() 
{	
	LATG = LATG & ~(0x0040);		/* clear SCK low */
}




/************************************************************************/
/* PROCEDURE: SI_High													*/
/*																		*/
/* This procedure set SI = High.										*/
/*																		*/
/* Input:																*/
/*		None															*/
/*																		*/
/* Output:																*/
/*		SI																*/
/*																		*/
/************************************************************************/
void SI_High() 
{
	LATG = LATG | (0x100);			/* set SIO0 high */
}

/************************************************************************/
/* PROCEDURE: SI_Low													*/
/*																		*/
/* This procedure drives the SI of the device to low. 		 			*/
/*																		*/
/* Input:																*/
/*		None															*/
/*																		*/
/* Output:																*/
/*		SI																*/
/*																		*/
/************************************************************************/
void SI_Low() 
{	
	LATG = LATG & ~(0x0100);		/* clear SIO0 low */
}


/************************************************************************/
/* PROCEDURE: SO_High													*/
/*																		*/
/* This procedure set SO = High.										*/
/*																		*/
/* Input:																*/
/*		None															*/
/*																		*/
/* Output:																*/
/*		SO																*/
/*																		*/
/************************************************************************/
void SO_High()
{
	LATG = LATG | (0x80);			/* set SIO1 high */
}

/************************************************************************/
/* PROCEDURE: SO_Low													*/
/*																		*/
/* This procedure drives the SO of the device to low. 		 			*/
/*																		*/
/* Input:																*/
/*		None															*/
/*																		*/
/* Output:																*/
/*		SO																*/
/*																		*/
/************************************************************************/
void SO_Low()
{
	LATG = LATG & ~(0x0080);		/* clear SIO1 low */
}

void SetSI_Output()
{
	TRISG = TRISG & ~(0x0100);
}
void SetSI_Input()
{
	TRISG = TRISG | (0x100);
}
void SetSO_Output()
{
	TRISG = TRISG & ~(0x0080);
}
void SetSO_Input()
{
	TRISG = TRISG | (0x0080);
}

				
					

/************************************************************************/
/* PROCEDURE: init														*/
/*																		*/
/* This procedure initializes the SCK to low. Must be called prior to 	*/
/* setting up mode 0.													*/
/*																		*/
/* Input:																*/
/*		None															*/
/*																		*/
/* Output:																*/
/*		SCK																*/
/************************************************************************/
void init()
{
	SCK_Low();	/* set clock to low initial state */
}

/************************************************************************/
/* PROCEDURE: Send_Byte													*/
/*																		*/
/* This procedure outputs a byte shifting out 1-bit per clock rising	*/
/* edge on the the SI pin(LSB 1st).										*/
/*																		*/
/* Input:																*/
/*		out																*/
/*																		*/
/* Output:																*/
/*		SI																*/
/************************************************************************/
void Send_Byte(unsigned char out)
{

	SetSI_Output();

		if ((out & 0x80) ==0x80)
			{SI_High();}
		else
			{SI_Low();}

		SCK_High();			/* toggle clock high */

		SCK_Low();			/* toggle clock low */

		if ((out & 0x40) ==0x40)
			{SI_High();}
		else
			{SI_Low();}

		SCK_High();			/* toggle clock high */

		SCK_Low();			/* toggle clock low */


		if ((out & 0x20) ==0x20)
			{SI_High();}
		else
			{SI_Low();}

		SCK_High();			/* toggle clock high */

		SCK_Low();			/* toggle clock low */


		if ((out & 0x10) ==0x10)
			{SI_High();}
		else
			{SI_Low();}

		SCK_High();			/* toggle clock high */

		SCK_Low();			/* toggle clock low */


		if ((out & 0x08) ==0x08)
			{SI_High();}
		else
			{SI_Low();}
		SCK_High();			/* toggle clock high */

		SCK_Low();			/* toggle clock low */


		if ((out & 0x04) ==0x04)
			{SI_High();}
		else
			{SI_Low();}

		SCK_High();			/* toggle clock high */

		SCK_Low();			/* toggle clock low */


		if ((out & 0x02) ==0x02)
			{SI_High();}
		else
			{SI_Low();}
		SCK_High();			/* toggle clock high */

		SCK_Low();			/* toggle clock low */


		if ((out & 0x01) ==0x01)
			{SI_High();}
		else
			{SI_Low();}

		SCK_High();			/* toggle clock high */

		SI_High();  //Set them as Inputs
		SO_High();

		SCK_Low();			/* toggle clock low */

}

/************************************************************************/
/* PROCEDURE: Send_Double_Byte						*/
/*									*/
/* This procedure outputs a byte shifting out 2-bit per clock rising	*/
/* edge on the the SI pin and SO pin(MSB 1st).				*/
/*									*/
/* Input:								*/
/*		out							*/
/*									*/
/* Output:								*/
/*		SI, SO							*/
/************************************************************************/
 void Send_Double_Byte(unsigned char out)
{

	SetSI_Output();
	SetSO_Output();

		if ((out & 0x80) ==0x80)
			{SO_High();}
		else
			{SO_Low();}

		if ((out & 0x40) ==0x40)
			{SI_High();}
		else
			{SI_Low();}

		SCK_High();			/* toggle clock high */

		SCK_Low();			/* toggle clock low */


		if ((out & 0x20) ==0x20)
			{SO_High();}
		else
			{SO_Low();}

		if ((out & 0x10) ==0x10)
			{SI_High();}
		else
			{SI_Low();}

		SCK_High();			/* toggle clock high */

		SCK_Low();			/* toggle clock low */


		if ((out & 0x08) ==0x08)
			{SO_High();}
		else
			{SO_Low();}

		if ((out & 0x04) ==0x04)
			{SI_High();}
		else
			{SI_Low();}

		SCK_High();			/* toggle clock high */

		SCK_Low();			/* toggle clock low */


		if ((out & 0x02) ==0x02)
			{SO_High();}
		else
			{SO_Low();}

		if ((out & 0x01) ==0x01)
			{SI_High();}
		else
			{SI_Low();}

		SCK_High();			/* toggle clock high */

		SI_High();  //Set them as Inputs
		SO_High();

		SCK_Low();			/* toggle clock low */


}

/************************************************************************/
/* PROCEDURE: Get_Byte													*/
/*																		*/
/* This procedure inputs a byte shifting in 1-bit per clock falling		*/
/* edge on the SO pin(LSB 1st).											*/
/*																		*/				
/* Input:																*/
/*		SO																*/
/*																		*/
/* Output:																*/
/*		None															*/
/************************************************************************/
unsigned char Get_Byte()
{
	unsigned char i = 0, in = 0, temp = 0;
        	SetSO_Input();
	for (i = 0; i < 8; i++)
	{
		in = (in << 1);		/* shift 1 place to the left or shift in 0 */
		temp = ((PORTG & 0x80)>>7);		/* save input */
		SCK_High();		/* toggle clock high */
	if (temp == 1)			/* check to see if bit is high */
		in = in | 0x01;		/* if high, make bit high */

		SCK_Low();		/* toggle clock low */

	}
	return in;
}


/************************************************************************/
/* PROCEDURE: Get_Double_Byte						*/
/*									*/
/* This procedure inputs a byte shifting in 2-bit per clock falling	*/
/* edge on the SO pin and SI pin(MSB 1st).				*/
/*									*/
/* Input:								*/
/*		SO, SI							*/
/*									*/
/* Output:								*/
/*		None							*/
/************************************************************************/
unsigned char Get_Double_Byte()
{
	unsigned char i = 0, in = 0, temp = 0, temp1=0;
	SetSI_Input();
	SetSO_Input();

	for (i = 0; i < 4; i++)
	{
		in = (in << 1);			/* shift 1 place to the left or shift in 0 */
			temp = ((PORTG & 0x80)>>7) ;	/* save input */
		temp1 = ((PORTG & 0x100)>>8);	/* save input */
			SCK_High();			/* toggle clock high */

		if (temp == 1)			/* check to see if bit is high */
		{in = in | 0x01;}		/* if high, make bit high */

		in=(in << 1);

		if (temp1 == 1)
		{in = in | 0x01;}

		SCK_Low();			/* toggle clock low */

	}
	return in;
}



/************************************************************************/
/* PROCEDURE: Read_Status_Register					*/
/*									*/
/* This procedure reads the status register and returns the byte.	*/
/************************************************************************/

unsigned char SPI_Read_Status_Register()
{    
	unsigned char byte = 0;
	CE_Low();			/* enable device */
	Send_Byte(0x05);		/* send RDSR command */
	byte = Get_Byte();		/* receive byte */
	CE_High();			/* disable device */
	return byte;
}

/************************************************************************/
/* PROCEDURE: WREN							*/
/*									*/
/* This procedure enables the Write Enable Latch.               	*/
/************************************************************************/

void SPI_WREN()
{   
	CE_Low();			/* enable device */
	Send_Byte(0x06);		/* send WREN command */
	CE_High();			/* disable device */
} 

/************************************************************************/
/* PROCEDURE: WRDI							*/
/*									*/
/* This procedure disables the Write Enable Latch.			*/
/************************************************************************/

void SPI_WRDI()
{   
	CE_Low();			/* enable device */
	Send_Byte(0x04);		/* send WRDI command */
	CE_High();			/* disable device */
}


/************************************************************************/
/* PROCEDURE: QuadJ_ID							*/
/*									*/
/* This procedure Reads the manufacturer's ID, device Type and device ID.  It will 	*/
/* use 9Fh as the command to read the ID.                               */
/* Returns:								*/
/*	ID1(Manufacture's Id, Device Type  , Device ID )	*/
/*									*/
/************************************************************************/

void Jedec_ID_Read()
{
   

	CE_Low();                            /* enable device */
	Send_Byte(0x9F);                    /* send JEDEC ID command (9Fh) */
   	Manufacturer_Id = Get_Byte();       /* receive byte */
  	Device_Type = Get_Byte();           /* receive byte */
  	Device_Id = Get_Byte();             /* receive byte */

	CE_High();                          /* disable device */
}

/************************************************************************/
/* PROCEDURE: Read_ID													*/
/*																		*/
/* This procedure Reads the device ID.  											*/
/*																		*/
/* Input:																*/
/*		ID_addr															*/
/*																		*/
/* Returns:																*/
/*		byte:	6Eh			*/
/*																		*/
/************************************************************************/
unsigned char Read_ID()
{
	unsigned char byte;
	CE_Low();			/* enable device */
	Send_Byte(0xAB);		/* send read ID command (90h or ABh) */
    	Send_Byte(0x00);		
	Send_Byte(0x00);		
	Send_Byte(0x00);
	byte = Get_Byte();		/* receive byte */
	CE_High();			/* disable device */
	return byte;
}

/************************************************************************/
/* PROCEDURE:	Read							*/
/*									*/
/* This procedure reads one address of the device.  It will return the 	*/
/* byte read in variable byte.						*/
/* Input:								*/
/*		Dst:	Destination Address 	*/
/************************************************************************/
unsigned char SPI_Read(unsigned long Dst)
{    
	unsigned char byte = 0;

	CE_Low();                                /* enable device */
	Send_Byte(0x03);                        /* read command */
	Send_Byte(((Dst & 0xFFFFFF) >> 16));	/* send 3 address bytes */
	Send_Byte(((Dst & 0xFFFF) >> 8));
	Send_Byte(Dst & 0xFF);
	byte = Get_Byte();
	CE_High();                              /* disable device */
	return byte;                            /* return one byte read */
}


/************************************************************************/
/* PROCEDURE:	Read_Cont						*/
/*									*/
/* This procedure reads multiple addresses of the device and stores	*/
/* data into 256 byte buffer. Maximum number of bytes read is limited 256 bytes*/
/*									*/
/* Input:								*/
/*		Dst:		Destination Address 	*/
/*      	no_bytes	Number of bytes to read	(max = 256)	*/
/************************************************************************/
void SPI_Read_Cont(unsigned long Dst, unsigned long no_bytes)
{//WOrks for both Chips
	unsigned long i = 0;
	CE_Low();				/* enable device */
	Send_Byte(0x03); 			/* read command */
	Send_Byte(((Dst & 0xFFFFFF) >> 16)); 	/* send 3 address bytes */
	Send_Byte(((Dst & 0xFFFF) >> 8));
	Send_Byte(Dst & 0xFF);
	if (no_bytes>256)
	{no_bytes=256;}
	for (i = 0; i < no_bytes; i++)		/* read until no_bytes is reached */
	{
		data_256[i] = Get_Byte();
	}
	CE_High();				/* disable device */

}

/************************************************************************/
/* PROCEDURE:	HighSpeed_Read						*/
/*									*/
/* This procedure reads one address of the device.  It will return the 	*/
/* byte read in variable byte.						*/
/* Input:								*/
/*		Dst:	Destination Address 		*/
/************************************************************************/
unsigned char SPI_HighSpeed_Read(unsigned long Dst)
{    
	unsigned char byte = 0;

	CE_Low();                               /* enable device */
	Send_Byte(0x0B);                        /* read command */
	Send_Byte(((Dst & 0xFFFFFF) >> 16));	/* send 3 address bytes */
	Send_Byte(((Dst & 0xFFFF) >> 8));
	Send_Byte(Dst & 0xFF);
	Send_Byte(0xFF);                        /*dummy byte*/
	byte = Get_Byte();
	CE_High();                              /* disable device */
	return byte;                            /* return one byte read */
}



/************************************************************************/
/* PROCEDURE:	HighSpeed_Read_Cont					*/
/*									*/
/* This procedure reads multiple addresses of the device and stores	*/
/* data into 256 byte buffer. Maximum number of bytes read is limited to 256 bytes*/
/*									*/
/* Input:								*/
/*		Dst:		Destination Address	*/
/*      	no_bytes	Number of bytes to read	(max = 256)	*/
/************************************************************************/
void SPI_HighSpeed_Read_Cont(unsigned long Dst, unsigned long no_bytes)
{    
	unsigned long i = 0;
	CE_Low();				/* enable device */
	Send_Byte(0x0B); 			/* read command */
	Send_Byte(((Dst & 0xFFFFFF) >> 16)); 	/* send 3 address bytes */
	Send_Byte(((Dst & 0xFFFF) >> 8));
	Send_Byte(Dst & 0xFF);
	Send_Byte(0xFF);                	/*dummy byte*/
	if (no_bytes>256)
	{no_bytes=256;}

	for (i = 0; i < no_bytes; i++)		/* read until no_bytes is reached */
	{
		data_256[i] = Get_Byte();
	}
	CE_High();				/* disable device */
}


/************************************************************************/
/* PROCEDURE:	SPI_Double_Output_Read					*/
/*									*/
/* This procedure reads one address of the device.  It will return the 	*/
/* byte read in variable byte.						*/
/* Input:								*/
/*		Dst:	Destination Address 		*/
/************************************************************************/
unsigned char SPI_Double_Output_Read(unsigned long Dst)
{    
	unsigned char byte = 0;

	CE_Low();			/* enable device */
	Send_Byte(0x3B); 		/* read command */
	Send_Byte(((Dst & 0xFFFFFF) >> 16));	/* send 3 address bytes */
	Send_Byte(((Dst & 0xFFFF) >> 8));
	Send_Byte(Dst & 0xFF);
	Send_Byte(0xFF);		/*dummy byte*/
	byte = Get_Double_Byte();
	CE_High();			/* disable device */
	return byte;			/* return one byte read */
}



/************************************************************************/
/* PROCEDURE:	SPI_Double_Output_Read_Cont				*/
/*									*/
/* This procedure reads multiple addresses of the device and stores	*/
/* data into 256 byte buffer. Maximum number of bytes read is limited to 256 bytes*/
/*									*/
/* Input:								*/
/*		Dst:		Destination Address 	*/
/*      	no_bytes	Number of bytes to read	(max = 256)	*/
/*									*/
/* Returns:								*/
/*		Nothing							*/
/*									*/
/************************************************************************/
void SPI_Double_Output_Read_Cont(unsigned long Dst, unsigned long no_bytes)
{    
	unsigned long i = 0;
	CE_Low();				/* enable device */
	Send_Byte(0x3B); 			/* read command */
	Send_Byte(((Dst & 0xFFFFFF) >> 16));	/* send 3 address bytes */
	Send_Byte(((Dst & 0xFFFF) >> 8));
	Send_Byte(Dst & 0xFF);
	Send_Byte(0xFF);                	/*dummy byte*/

	if (no_bytes>256)
	{no_bytes=256;}

	for (i = 0; i < no_bytes; i++)		/* read until no_bytes is reached */
	{
		data_256[i] = Get_Double_Byte();
	}
	CE_High();				/* disable device */
}

/************************************************************************/
/* PROCEDURE:	SPI_Double_IO_Read					*/
/*									*/
/* This procedure reads one address of the device.  It will return the 	*/
/* byte read in variable byte.						*/
/* Input:								*/
/*		Dst:	Destination Address  		*/
/************************************************************************/
unsigned char SPI_Double_IO_Read(unsigned long Dst)
{    
	unsigned char byte = 0;

	CE_Low();			/* enable device */
	Send_Byte(0xBB); 		/* read command */
	Send_Double_Byte(((Dst & 0xFFFFFF) >> 16));	/* send 3 address bytes */
	Send_Double_Byte(((Dst & 0xFFFF) >> 8));
	Send_Double_Byte(Dst & 0xFF);
	Send_Double_Byte(0xFF);		/*dummy byte*/
	byte = Get_Double_Byte();
	CE_High();			/* disable device */
	return byte;			/* return one byte read */
}


/************************************************************************/
/* PROCEDURE:	SPI_Double_IO_Read_Cont					*/
/*									*/
/* This procedure reads multiple addresses of the device and stores	*/
/* data into 256 byte buffer. Maximum number of bytes read is limited to 256 bytes*/
/*									*/
/* Input:								*/
/*		Dst:		Destination Address  	*/
/*      	no_bytes	Number of bytes to read	(max = 256)	*/
/************************************************************************/
void SPI_Double_IO_Read_Cont(unsigned long Dst, unsigned long no_bytes)
{    
	unsigned long i=0;
	CE_Low();                                       /* enable device */
	Send_Byte(0xBB);                                /* read command */
	Send_Double_Byte(((Dst & 0xFFFFFF) >> 16));	/* send 3 address bytes */
	Send_Double_Byte(((Dst & 0xFFFF) >> 8));
	Send_Double_Byte(Dst & 0xFF);
	Send_Double_Byte(0xFF);                         /*dummy byte*/

	if (no_bytes>256)
	{no_bytes=256;}

	for ( i = 0; i < no_bytes; i++)                 /* read until no_bytes is reached */
	{
		data_256[i] = Get_Double_Byte();
	}
	CE_High();                                      /* disable device */
}





/************************************************************************/
/* PROCEDURE:	Page_Program						*/
/*									*/
/* This procedure does page programming.  The destination               */
/* address should be provided.                                  	*/
/* The data array of 256 bytes contains the data to be programmed.      */
/* Since the size of the data array is 256 bytes rather than 256 bytes, this page program*/
/* procedure programs only 256 bytes                                    */
/* Assumption:  Address being programmed is already erased and is NOT	*/
/*		block protected.					*/
/* Input:								*/
/*		Dst:		Destination Address  	*/
/*		data_256[256] containing 256 bytes of data will be programmed using this function */
/************************************************************************/

void SPI_Page_Program(unsigned long Dst)
{    
	unsigned int i;
	i=0;

	CE_Low();				/* enable device */
	Send_Byte(0x02); 			/* send Byte Program command */
	Send_Byte(((Dst & 0xFFFFFF) >> 16));	/* send 3 address bytes */
	Send_Byte(((Dst & 0xFFFF) >> 8));
	Send_Byte(Dst & 0xFF);
	for (i=0;i<256;i++)
	{	Send_Byte(data_256[i]);		/* send byte to be programmed */
	}
	CE_High();				/* disable device */
}



/************************************************************************/
/* PROCEDURE: Chip_Erase						*/
/*									*/
/* This procedure erases the entire Chip.				*/
/************************************************************************/


void SPI_Chip_Erase()
{    
	CE_Low();				/* enable device */
	Send_Byte(0xC7);			/* send Chip Erase command (C7h) */
	CE_High();				/* disable device */
}

/************************************************************************/
/* PROCEDURE: Sector_Erase						*/
/*									*/
/* This procedure Sector Erases the Chip.				*/
/* Input:								*/
/*		Dst:		Destination Address  	*/
/************************************************************************/


void SPI_Sector_Erase(unsigned long Dst)
{
    

	CE_Low();				/* enable device */
	Send_Byte(0x20);			/* send Sector Erase command */
	Send_Byte(((Dst & 0xFFFFFF) >> 16)); 	/* send 3 address bytes */
	Send_Byte(((Dst & 0xFFFF) >> 8));
	Send_Byte(Dst & 0xFF);
	CE_High();				/* disable device */
}

/************************************************************************/
/* PROCEDURE: Block_Erase						*/
/*									*/
/* This procedure Block Erases 8Kbyte, 32 KByte or 64 KByte of the Chip.*/
/*									*/
/* Input:								*/
/*		Dst:		Destination Address  	*/
/************************************************************************/

void SPI_Block_Erase(unsigned long Dst)
{    
	CE_Low();				/* enable device */
	Send_Byte(0xD8);			/* send Block Erase command */
	Send_Byte(((Dst & 0xFFFFFF) >> 16)); 	/* send 3 address bytes */
	Send_Byte(((Dst & 0xFFFF) >> 8));
	Send_Byte(Dst & 0xFF);
	CE_High();				/* disable device */
}

/************************************************************************/
/* PROCEDURE: DPD														*/
/*																		*/
/* This procedure causes part to enter deep power down.									*/
/*																		*/
/* Input:																*/
/*		None															*/
/*																		*/
/* Returns:																*/
/*		Nothing															*/
/************************************************************************/
void SPI_DPD()
{
	CE_Low();			/* enable device */
	Send_Byte(0xB9);		/* send WREN command */
	CE_High();			/* disable device */
}

/************************************************************************/
/* PROCEDURE: RDPD														*/
/*																		*/
/* This procedure enables the part to release from deep power down.									*/
/*																		*/
/* Input:																*/
/*		None															*/
/*																		*/
/* Returns:																*/
/*		Nothing															*/
/************************************************************************/
void SPI_RDPD()
{
	CE_Low();			/* enable device */
	Send_Byte(0xAB);		/* send WREN command */
	CE_High();			/* disable device */
}


/************************************************************************/
/* PROCEDURE: NoOp                                              	*/
/*									*/
/* No operation is performed.                                           */
/************************************************************************/

void SPI_NoOp()
{
	CE_Low();				/* enable device */
	Send_Byte(0x00);
	CE_High();				/* disable device */
}




/************************************************************************/
/* PROCEDURE: Write_Status_Register					*/
/*									*/
/* This procedure resumes Program/Erase operation.			*/
/************************************************************************/

void SPI_Write_Status_Register(unsigned int data1)
{	
    CE_Low();				/* enable device */
	Send_Byte(0x01);
   
	Send_Byte((data1)&0xff);
	
  
	CE_High();				/* disable device */
}


/************************************************************************/
/* PROCEDURE: Wait_Busy							*/
/*									*/
/* This procedure waits until device is no longer busy (can be used by	*/
/* Byte-Program, Page-Program, Sector-Erase, Block-Erase and Chip-Erase).*/
/************************************************************************/



void SPI_Wait_Busy()
{    
	while ((SPI_Read_Status_Register()& 0x01) == 0x01)	// waste time until not busy
		SPI_Read_Status_Register();
}

 /****************************************************/
 /* Main Function*/
 /*****************************************************/

int main()
{

unsigned long i,j,k,highest_address;
int check,tempcheck,status;
unsigned char Led_Stat;

int n;
n=0;
   i=0;
 j=0;
 Led_Stat=0x00;
 status=1;  //1 means memory/code works and 0 means fault has occured.
 check=0;  //keeps track of code progress.
 tempcheck=1;
highest_address=0x1FFFF;
 
AD1PCFG= AD1PCFG|0x02; //To make SIO3, RB1 a digital pin.



Portassign();
CE_High();
WP_High();
Hold_High();

 init();

 	i=0;
while (i<=0xFFFF)
{
DisplayLED(0xFF);
	i++;
}
DisplayLED(0x00);

// Test Jedec ID
      
/************* 1a. Test Jedec ID *********/
 
Jedec_ID_Read();

 if ((Manufacturer_Id == 0x62)&& (Device_Type==0x06) && (Device_Id==0x13)&&(status==1))
{check=(check + 1);
  //highest_address=0x7FFFF;
 }
else
{check=0;
status=0;}

 DisplayLED(check);
 					
 
 		
/************* 2a. Page program whole chip using SPI protocol and verify its OK. *********/

 	i=0;
while (i<=255)
{
 data_256[i]=170;
	i++;
}
 	
        SPI_WREN();
        SPI_Write_Status_Register(0x00);
  SPI_Wait_Busy();
    
    SPI_WREN();
 	
SPI_Chip_Erase();	//Erase the chip


SPI_Wait_Busy();
 i=0;
SPI_WREN();
		

 while(i<highest_address)
 {	SPI_WREN();
DisplayLED(5);
SPI_Page_Program(i);
	SPI_Wait_Busy();
	i=i+256;
}

	  	SPI_WREN();


  j=0;
 while(j<highest_address)
 {
	

	   SPI_HighSpeed_Read_Cont(j, 64);
		 for (i=0;i<64;i++)
		 {
		 	if (data_256[i]==170)
			{tempcheck&=1;
			}
			  else
			{ tempcheck=0;
			}
		  }

	j=j+64;
}


	  	if ((tempcheck == 1)&&(status==1))
		{check=(check+1);
		status=status&1;}
		else
		{status=0;}


	DisplayLED(check);


/************* 3a. Do Sector Erase and verify that the sector is erased *********/


	SPI_WREN();
	SPI_Sector_Erase(0);	//Do Sector Erase
	SPI_Wait_Busy();

        j=0;
 	while (j<(4096))
	{
    	SPI_WREN();
  	  SPI_HighSpeed_Read_Cont(j, 64);
	    for (k=0;k<63;k++)
		 {
		 	if (data_256[k]==0xff)			  // Verify that the values are correct
			{tempcheck&=1;}
			  else
			{ tempcheck=0;}
   		  }
   		  j=j+64;
	}

        j=4096;
 	while (j<(4096*2))
	{
    	SPI_WREN();
    SPI_Double_Output_Read_Cont(j, 64);
	    for (k=0;k<63;k++)
		 {
		 	if (data_256[k]==0xAA)			  // Verify that the values are correct
			{tempcheck&=1;}
			  else
			{ tempcheck=0;}
   		  }
   		  j=j+64;
	}


	  	if ((tempcheck == 1)&&(status==1))
		{check=(check+2);
		}
		else
		{status=0;}


	DisplayLED(check);

/************* 4a. Block Erase and verify that the Block is erased *********/

	SPI_WREN();
	SPI_Block_Erase(0);	//Do Block Erase
	SPI_Wait_Busy();

        j=0;
 	while (j<(0x2000))
	{
    	SPI_WREN();
    SPI_HighSpeed_Read_Cont(j, 64);
	    for (k=0;k<63;k++)
		 {
		 	if (data_256[k]==0xff)			  // Verify that the values are correct
			{tempcheck&=1;}
			  else
			{ tempcheck=0;}
   		  }
   		  j=j+64;
	}
      
                j=0x10000;
 	while (j<(0x20000))
	{
    	SPI_WREN();
   SPI_Double_IO_Read_Cont(j, 64);
		    for (k=0;k<63;k++)
		 {
		 	if (data_256[k]==0xAA)			  // Verify that the values are correct
			{tempcheck&=1;}
			  else
			{ tempcheck=0;}
   		  }
   		  j=j+64;
	}
        

	  	if ((tempcheck == 1)&&(status==1))
		{check=(check+1);
		}
		else
		{status=0;
		}



	DisplayLED(check);

/************* 5a. Chip Erase and Verify Chip is erased*********/

	SPI_WREN();
	SPI_Chip_Erase();	//Do Sector Erase
	SPI_Wait_Busy();

        j=0;
 	while (j<highest_address)
	{
    	SPI_WREN();
   SPI_HighSpeed_Read_Cont(j, 64);
		    for (k=0;k<63;k++)
		 {
		 	if (data_256[k]==0xFF)			  // Verify that the values are correct
			{tempcheck&=1;}
			  else
			{ tempcheck=0;}
   		  }
   		  j=j+64;
	}

	  	if ((tempcheck == 1)&&(status==1))
		{check=(check+3);
		}
		else
		{status=0;}



	DisplayLED(check);
        
 /***************** End of checking ******/


  Led_Stat=check;
  if (status==0) //means a failure has occur
  {
   	Led_Stat|=0x80;
  }

 while(1)


{
	DisplayLED(Led_Stat);
}

return 0;
}













