
#include "SST25PF040C_functions.h"

//unsigned int data_256[256];	/* global array to store read data */
//unsigned char   Manufacturer_Id, Device_Type, Device_Id; 

                                                          
unsigned char SPI_Read_Status_Register()
{    
	unsigned char byte = 0;
	CE_SetLow();			/* enable device */
	SPI1_Exchange8bit(0x05);		/* send RDSR command */
	byte = SPI1_Exchange8bit(0xFF);		/* receive byte */
	CE_SetHigh();			/* disable device */
	return byte;
}

/************************************************************************/
/* PROCEDURE: WREN							*/
/*									*/
/* This procedure enables the Write Enable Latch.               	*/
/************************************************************************/

void SPI_WREN()
{   
	CE_SetLow();			/* enable device */
	SPI1_Exchange8bit(0x06);		/* send WREN command */
	CE_SetHigh();			/* disable device */
} 

/************************************************************************/
/* PROCEDURE: WRDI							*/
/*									*/
/* This procedure disables the Write Enable Latch.			*/
/************************************************************************/

void SPI_WRDI()
{   
	CE_SetLow();			/* enable device */
	SPI1_Exchange8bit(0x04);		/* send WRDI command */
	CE_SetHigh();			/* disable device */
}


/************************************************************************/
/* PROCEDURE: QuadJ_ID							*/
/*									*/
/* This procedure Reads the manufacturer's ID, device Type and device ID.  It will 	*/
/* use 9Fh as the command to read the ID.                               */
/* Returns:								*/
/*	ID1(Manufacture's Id, Device Type  , Device ID )	*/
/*									*/
/************************************************************************/

void Jedec_ID_Read(char *Manufacturer_Id, char *Device_Type, char *Device_Id)
{
   

	CE_SetLow();                            /* enable device */
	SPI1_Exchange8bit(0x9F);                    /* send JEDEC ID command (9Fh) */
   	*Manufacturer_Id = SPI1_Exchange8bit(0xFF);       /* receive byte */
  	*Device_Type = SPI1_Exchange8bit(0xFF);           /* receive byte */
  	*Device_Id = SPI1_Exchange8bit(0xFF);             /* receive byte */

	CE_SetHigh();                          /* disable device */
}

/************************************************************************/
/* PROCEDURE: Read_ID													*/
/*																		*/
/* This procedure Reads the device ID.  											*/
/*																		*/
/* Input:																*/
/*		ID_addr															*/
/*																		*/
/* Returns:																*/
/*		byte:	6Eh			*/
/*																		*/
/************************************************************************/
unsigned char Read_ID()
{
	unsigned char byte;
	CE_SetLow();			/* enable device */
	SPI1_Exchange8bit(0xAB);		/* send read ID command (90h or ABh) */
    	SPI1_Exchange8bit(0x00);		
	SPI1_Exchange8bit(0x00);		
	SPI1_Exchange8bit(0x00);
	byte = SPI1_Exchange8bit(0xFF);		/* receive byte */
	CE_SetHigh();			/* disable device */
	return byte;
}

/************************************************************************/
/* PROCEDURE:	Read							*/
/*									*/
/* This procedure reads one address of the device.  It will return the 	*/
/* byte read in variable byte.						*/
/* Input:								*/
/*		Dst:	Destination Address 	*/
/************************************************************************/
unsigned char SPI_Read(unsigned long Dst)
{    
	unsigned char byte = 0;

	CE_SetLow();                                /* enable device */
	SPI1_Exchange8bit(0x03);                        /* read command */
	SPI1_Exchange8bit(((Dst & 0xFFFFFF) >> 16));	/* send 3 address bytes */
	SPI1_Exchange8bit(((Dst & 0xFFFF) >> 8));
	SPI1_Exchange8bit(Dst & 0xFF);
	byte = SPI1_Exchange8bit(0xFF);
	CE_SetHigh();                              /* disable device */
	return byte;                            /* return one byte read */
}


/************************************************************************/
/* PROCEDURE:	Read_Cont						*/
/*									*/
/* This procedure reads multiple addresses of the device and stores	*/
/* data into 256 byte buffer. Maximum number of bytes read is limited 256 bytes*/
/*									*/
/* Input:								*/
/*		Dst:		Destination Address 	*/
/*      	no_bytes	Number of bytes to read	(max = 256)	*/
/************************************************************************/
void SPI_Read_Cont(unsigned long Dst,unsigned int *read_data, unsigned long no_bytes)
{//WOrks for both Chips
	unsigned long i = 0;
	CE_SetLow();				/* enable device */
	SPI1_Exchange8bit(0x03); 			/* read command */
	SPI1_Exchange8bit(((Dst & 0xFFFFFF) >> 16)); 	/* send 3 address bytes */
	SPI1_Exchange8bit(((Dst & 0xFFFF) >> 8));
	SPI1_Exchange8bit(Dst & 0xFF);
	if (no_bytes>256)
	{no_bytes=256;}
	for (i = 0; i < no_bytes; i++)		/* read until no_bytes is reached */
	{
		*read_data = SPI1_Exchange8bit(0xFF);
        read_data++;
	}
	CE_SetHigh();				/* disable device */

}

/************************************************************************/
/* PROCEDURE:	HighSpeed_Read						*/
/*									*/
/* This procedure reads one address of the device.  It will return the 	*/
/* byte read in variable byte.						*/
/* Input:								*/
/*		Dst:	Destination Address 		*/
/************************************************************************/
unsigned char SPI_HighSpeed_Read(unsigned long Dst)
{    
	unsigned char byte = 0;

	CE_SetLow();                               /* enable device */
	SPI1_Exchange8bit(0x0B);                        /* read command */
	SPI1_Exchange8bit(((Dst & 0xFFFFFF) >> 16));	/* send 3 address bytes */
	SPI1_Exchange8bit(((Dst & 0xFFFF) >> 8));
	SPI1_Exchange8bit(Dst & 0xFF);
	SPI1_Exchange8bit(0xFF);                        /*dummy byte*/
	byte = SPI1_Exchange8bit(0xFF);
	CE_SetHigh();                              /* disable device */
	return byte;                            /* return one byte read */
}



/************************************************************************/
/* PROCEDURE:	HighSpeed_Read_Cont					*/
/*									*/
/* This procedure reads multiple addresses of the device and stores	*/
/* data into 256 byte buffer. Maximum number of bytes read is limited to 256 bytes*/
/*									*/
/* Input:								*/
/*		Dst:		Destination Address	*/
/*      	no_bytes	Number of bytes to read	(max = 256)	*/
/************************************************************************/
void SPI_HighSpeed_Read_Cont(unsigned long Dst,unsigned int *read_data, unsigned long no_bytes)
{    
	unsigned long i = 0;
	CE_SetLow();				/* enable device */
	SPI1_Exchange8bit(0x0B); 			/* read command */
	SPI1_Exchange8bit(((Dst & 0xFFFFFF) >> 16)); 	/* send 3 address bytes */
	SPI1_Exchange8bit(((Dst & 0xFFFF) >> 8));
	SPI1_Exchange8bit(Dst & 0xFF);
	SPI1_Exchange8bit(0xFF);                	/*dummy byte*/
	if (no_bytes>256)
	{no_bytes=256;}

	for (i = 0; i < no_bytes; i++)		/* read until no_bytes is reached */
	{
		*read_data = SPI1_Exchange8bit(0xFF);
        read_data++;
	}
	CE_SetHigh();				/* disable device */
}




/************************************************************************/
/* PROCEDURE:	Page_Program						*/
/*									*/
/* This procedure does page programming.  The destination               */
/* address should be provided.                                  	*/
/* The data array of 256 bytes contains the data to be programmed.      */
/* Since the size of the data array is 256 bytes rather than 256 bytes, this page program*/
/* procedure programs only 256 bytes                                    */
/* Assumption:  Address being programmed is already erased and is NOT	*/
/*		block protected.					*/
/* Input:								*/
/*		Dst:		Destination Address  	*/
/*		data_256[256] containing 256 bytes of data will be programmed using this function */
/************************************************************************/

void SPI_Page_Program(unsigned long Dst, unsigned int *Prog_data)
{    
	unsigned int i;
	i=0;

	CE_SetLow();				/* enable device */
	SPI1_Exchange8bit(0x02); 			/* send Byte Program command */
	SPI1_Exchange8bit(((Dst & 0xFFFFFF) >> 16));	/* send 3 address bytes */
	SPI1_Exchange8bit(((Dst & 0xFFFF) >> 8));
	SPI1_Exchange8bit(Dst & 0xFF);
	for (i=0;i<256;i++)
	{	SPI1_Exchange8bit(*Prog_data);		/* send byte to be programmed */
	Prog_data++;
    }
	CE_SetHigh();				/* disable device */
}



/************************************************************************/
/* PROCEDURE: Chip_Erase						*/
/*									*/
/* This procedure erases the entire Chip.				*/
/************************************************************************/


void SPI_Chip_Erase()
{    
	CE_SetLow();				/* enable device */
	SPI1_Exchange8bit(0xC7);			/* send Chip Erase command (C7h) */
	CE_SetHigh();				/* disable device */
}

/************************************************************************/
/* PROCEDURE: Sector_Erase						*/
/*									*/
/* This procedure Sector Erases the Chip.				*/
/* Input:								*/
/*		Dst:		Destination Address  	*/
/************************************************************************/


void SPI_Sector_Erase(unsigned long Dst)
{
    

	CE_SetLow();				/* enable device */
	SPI1_Exchange8bit(0x20);			/* send Sector Erase command */
	SPI1_Exchange8bit(((Dst & 0xFFFFFF) >> 16)); 	/* send 3 address bytes */
	SPI1_Exchange8bit(((Dst & 0xFFFF) >> 8));
	SPI1_Exchange8bit(Dst & 0xFF);
	CE_SetHigh();				/* disable device */
}

/************************************************************************/
/* PROCEDURE: Block_Erase						*/
/*									*/
/* This procedure Block Erases 8Kbyte, 32 KByte or 64 KByte of the Chip.*/
/*									*/
/* Input:								*/
/*		Dst:		Destination Address  	*/
/************************************************************************/

void SPI_Block_Erase(unsigned long Dst)
{    
	CE_SetLow();				/* enable device */
	SPI1_Exchange8bit(0xD8);			/* send Block Erase command */
	SPI1_Exchange8bit(((Dst & 0xFFFFFF) >> 16)); 	/* send 3 address bytes */
	SPI1_Exchange8bit(((Dst & 0xFFFF) >> 8));
	SPI1_Exchange8bit(Dst & 0xFF);
	CE_SetHigh();				/* disable device */
}

/************************************************************************/
/* PROCEDURE: DPD														*/
/*																		*/
/* This procedure causes part to enter deep power down.									*/
/*																		*/
/* Input:																*/
/*		None															*/
/*																		*/
/* Returns:																*/
/*		Nothing															*/
/************************************************************************/
void SPI_DPD()
{
	CE_SetLow();			/* enable device */
	SPI1_Exchange8bit(0xB9);		/* send WREN command */
	CE_SetHigh();			/* disable device */
}

/************************************************************************/
/* PROCEDURE: RDPD														*/
/*																		*/
/* This procedure enables the part to release from deep power down.									*/
/*																		*/
/* Input:																*/
/*		None															*/
/*																		*/
/* Returns:																*/
/*		Nothing															*/
/************************************************************************/
void SPI_RDPD()
{
	CE_SetLow();			/* enable device */
	SPI1_Exchange8bit(0xAB);		/* send WREN command */
	CE_SetHigh();			/* disable device */
}


/************************************************************************/
/* PROCEDURE: NoOp                                              	*/
/*									*/
/* No operation is performed.                                           */
/************************************************************************/

void SPI_NoOp()
{
	CE_SetLow();				/* enable device */
	SPI1_Exchange8bit(0x00);
	CE_SetHigh();				/* disable device */
}




/************************************************************************/
/* PROCEDURE: Write_Status_Register					*/
/*									*/
/* This procedure resumes Program/Erase operation.			*/
/************************************************************************/

void SPI_Write_Status_Register(unsigned int data1)
{	
    CE_SetLow();				/* enable device */
	SPI1_Exchange8bit(0x01);
   
	SPI1_Exchange8bit((data1)&0xff);
	
  
	CE_SetHigh();				/* disable device */
}


/************************************************************************/
/* PROCEDURE: Wait_Busy							*/
/*									*/
/* This procedure waits until device is no longer busy (can be used by	*/
/* Byte-Program, Page-Program, Sector-Erase, Block-Erase and Chip-Erase).*/
/************************************************************************/



void SPI_Wait_Busy()
{    
	while ((SPI_Read_Status_Register()& 0x01) == 0x01)	// waste time until not busy
		SPI_Read_Status_Register();
}
